/*
    zr36120.c - Zoran 36120/36125 based framegrabbers

    Copyright (C) 1998-1999 Pauline Middelink <middelink@polyware.nl>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <linux/module.h>
#include <linux/delay.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/malloc.h>
#include <linux/vmalloc.h>
#include <linux/mm.h>
#include <linux/pci.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <asm/io.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/segment.h>

#include <linux/version.h>
#include <asm/uaccess.h>

#include "linux/video_decoder.h"
#include "linux/tuner.h"
#include "zr36120.h"
#include "zr36120_mem.h"

/* mark an required function argument unused - lintism */
#define	UNUSED(x)	(void)(x)

#if LINUX_VERSION_CODE < 0x02020c
#error zoran driver requires kernel 2.2.12 or better
#endif

/* sensible default */
#ifndef CARDTYPE
#define CARDTYPE 0
#endif

/* Anybody who uses more than four? */
#define ZORAN_MAX 4

static uint triton1;			/* triton1 chipset? */
static uint cardtype[ZORAN_MAX]={ [ 0 ... ZORAN_MAX-1 ] = CARDTYPE };
static uint nrcaps[ZORAN_MAX]={ [ 0 ... ZORAN_MAX-1 ] = ZORAN_CAP_FRAMES };
static uint nrvbis[ZORAN_MAX]={ [ 0 ... ZORAN_MAX-1 ] = ZORAN_VBI_FRAMES };
static uint szcap[ZORAN_MAX]={ [ 0 ... ZORAN_MAX-1 ] = ZORAN_CAP_BUFSIZE };

MODULE_AUTHOR("Pauline Middelink <middelink@polyware.nl>");
MODULE_DESCRIPTION("Zoran ZR36120 based framegrabber");
MODULE_PARM(triton1,"i");
MODULE_PARM(cardtype,"1-" __MODULE_STRING(ZORAN_MAX) "i");
MODULE_PARM(nrcaps,"1-" __MODULE_STRING(ZORAN_MAX) "i");
MODULE_PARM(nrvbis,"1-" __MODULE_STRING(ZORAN_MAX) "i");
MODULE_PARM(szcap,"1-" __MODULE_STRING(ZORAN_MAX) "i");

#if LINUX_VERSION_CODE < 0x020314
static unsigned int remap[ZORAN_MAX];	/* remap Zoran */
MODULE_PARM(remap,"1-" __MODULE_STRING(ZORAN_MAX) "i");
#endif

static int zoran_cards;
static struct zoran zorans[ZORAN_MAX];

/*
 * the meaning of each element can be found in zr36120.h
 * Determining the value of gpdir/gpval can be tricky. The
 * best way is to run the card under the original software
 * and read the values from the general purpose registers
 * 0x28 and 0x2C. How you do that is left as an exercise
 * to the impatient reader :)
 */
#define T 1	/* to seperate the bools from the ints */
#define F 0
static struct tvcard tvcards[] = {
	/* reported working by <middelink@polyware.nl> */
/*0*/	{ "Trust Victor II",
	  2, 0, F, T, T, T, T, F, 0x7F, 0x80, { 1, SVHS(6) }, { 0 } },

	/* reported working by <Michael.Paxton@aihw.gov.au>  */
/*1*/   { "Aitech WaveWatcher TV-PCI",
	  3, 0, F, F, T, T, T, F, 0x7F, 0x80, { 1, TUNER(3), SVHS(6) }, { 0 } },

	/* reported working by ? */
/*2*/	{ "Genius Video Wonder",
	  2, 0, F, T, T, T, T, F, 0x7F, 0x80, { 1, SVHS(6) }, { 0 } },

	/* reported working by <Pascal.Gabriel@wanadoo.fr> */
/*3*/	{ "Guillemot Maxi-TV PCI",
	  2, 0, F, T, T, T, T, F, 0x7F, 0x80, { 1, SVHS(6) }, { 0 } },

	/* reported working by "Craig Whitmore <lennon@igrin.co.nz> */
/*4*/	{ "Quadrant Buster",
	  3, 3, F, F, T, T, T, F, 0x7F, 0x80, { SVHS(1), TUNER(2), 3 }, { 1, 2, 3 } },

	/* reported by <Peter.Schlaf@org.chemie.uni-giessen.de> */
/*5*/	{ "MediaFocus Satellite TV Card",
	  3, 0, T, F, T, F, T, F, 0xE0, 0x11, { 0, TUNER(2), SVHS(4) }, { 0 } },

	/* reported working by <rfassano@teletel.com.ar> */
/*6*/	{ "Genius TV Wonder Pro",
	  3, 1, F, T, T, T, T, F, 0x7F, 0x80, { 1, TUNER(2), SVHS(3) }, { 0 } },

	/* reported working by Joel Fuster <j@fuster.org> */
/*7*/	{ "Boca BVPSZ1",
	  1, 0, F, T, T, F, T, F, 0xFD, 0x00, { 1 }, { 0 } },

	/* debug card, all inputs enabled */
/*8*/	{ "ZR36120 based framegrabber (all inputs enabled)",
	  6, 0, F, T, T, T, T, F, 0x7F, 0x80, { 1, 2, 3, 4, 5, 6 }, { 0 } }
};
#undef T
#undef F
#define NRTVCARDS (sizeof(tvcards)/sizeof(tvcards[0]))

#ifdef __sparc__
#define	ENDIANESS	0
#define	ENDIANES_	ZORAN_VFEC_LE
#else
#define	ENDIANESS	ZORAN_VFEC_LE
#define	ENDIANES_	0
#endif

static struct { const char name[8]; uint mode; uint bpp; } palette2fmt[] = {
/* n/a     */	{ "n/a",     0, 0 },
/* GREY    */	{ "GRAY",    0, 0 },
/* HI240   */	{ "HI240",   0, 0 },
/* RGB565  */	{ "RGB565",  ZORAN_VFEC_RGB_RGB565|ENDIANESS, 2 },
/* RGB24   */	{ "RGB24",   ZORAN_VFEC_RGB_RGB888|ENDIANESS|ZORAN_VFEC_PACK24, 3 },
/* RGB32   */	{ "RGB32",   ZORAN_VFEC_RGB_RGB888|ENDIANESS, 4 },
/* RGB555  */	{ "RGB555",  ZORAN_VFEC_RGB_RGB555|ENDIANESS, 2 },
/* YUV422  */	{ "YUV422",  0, 0 },
/* YUYV    */	{ "YUYV",    ZORAN_VFEC_RGB_YUV422|ENDIANES_, 2 },
/* UYVY    */	{ "UYVY",    ZORAN_VFEC_RGB_YUV422|ENDIANESS, 2 },
/* YUV420  */	{ "YUV420",  0, 0 },
/* YUV411  */	{ "YUV411",  0, 0 },
/* RAW     */	{ "RAW",     0, 0 },
/* YUV422P */	{ "YUV422P", 0, 0 },
/* YUV411P */	{ "YUV411P", 0, 0 }};
#define NRPALETTES (sizeof(palette2fmt)/sizeof(palette2fmt[0]))
#undef ENDIANESS
#undef ENDIANES_

/* ----------------------------------------------------------------------- */
/* ZORAN functions							   */
/* ----------------------------------------------------------------------- */

static void zoran_set_geo(struct zoran* ztv, struct vidinfo* i);

/**
 * reap_states() - process the ongoing VSYNC interrupt
 * @ztv: zoran card we are working with
 *
 * Does reap stuff using @ztv.
 **/
static
void reap_states(struct zoran* ztv)
{
	/*
	 * Are we busy at all?
	 * This depends on if there is a workqueue AND the
	 * videotransfer is enabled on the chip...
	 */
	if (!list_empty(&ztv->workqueue) &&
	    (zrread(ZORAN_VDC) & ZORAN_VDC_VIDEN))
	{
		struct vidinfo* work = list_entry(ztv->workqueue.next, struct vidinfo, work_item);
#if FDEBUG
DEBUG(printk(CARD_DEBUG "tfield=%08lx, bfield=%08lx\n",CARD,*work->tfield,*work->bfield));

		if (*work->tfield == 0x12345678UL && *work->bfield != 0x87654321UL)
		{
DEBUG(printk(CARD_DEBUG "We missed a topfield. Retry\n",CARD));
			*work->bfield = 0x87654321UL;

			/* stop the chip */
			zrand(~ZORAN_VDC_VIDEN,ZORAN_VDC);
			zrand(~ZORAN_VSTR_GRAB, ZORAN_VSTR);

			/* and restart it again */
			zror(ZORAN_VDC_VIDEN,ZORAN_VDC);
			zror(ZORAN_VSTR_GRAB, ZORAN_VSTR);
			return;
		}
#endif

		/* did we get a complete frame? */
		if (zrread(ZORAN_VSTR) & ZORAN_VSTR_GRAB)
			return;

		/* we are done with this buffer, tell everyone */
		work->loaded = 1;
		work->fieldnr = ztv->fieldnr;
		list_del(&work->work_item);

		switch (work->kindof) {
		 case FBUFFER_CAP:
DEBUG(printk(CARD_DEBUG "completed cap-%d\n",CARD,work - ztv->capinfo));
			wake_up_interruptible(&ztv->capcq);
			break;
		 case FBUFFER_VBI:
DEBUG(printk(CARD_DEBUG "completed vbi-%d\n",CARD,work - ztv->capinfo));
			wake_up_interruptible(&ztv->vbicq);
			break;
		 default:
			printk(CARD_INFO "somebody killed the workqueue (kindof=%d)!\n",CARD,work->kindof);
		}
	}

	/*
	 * ok, so it seems we have nothing in progress right now.
	 * We have however found a new workitem, so start it up.
	 */
	if (!list_empty(&ztv->workqueue))
	{
		struct vidinfo* work;
again:
		work = list_entry(ztv->workqueue.next, struct vidinfo, work_item);
DEBUG(switch (work->kindof) {
	case FBUFFER_CAP:
		printk(CARD_DEBUG "starting cap-%d\n",CARD,work - ztv->capinfo);
		break;
	case FBUFFER_VBI:
		printk(CARD_DEBUG "starting vbi-%d\n",CARD,work - ztv->vbiinfo);
		break;
	default:
		printk(CARD_DEBUG "what's in the queue <??>",CARD);
	}
);

		/* loadup the frame settings */
		read_lock(&ztv->lock);
		zoran_set_geo(ztv,work);
#if FDEBUG
*(work->tfield) = 0x12345678UL;
*(work->bfield) = 0x87654321UL;
#endif
		read_unlock(&ztv->lock);

		switch (work->kindof) {
		 case FBUFFER_CAP:
		 case FBUFFER_VBI:
			zrand(~ZORAN_OCR_OVLEN, ZORAN_OCR);
			zror(ZORAN_VSTR_SNAPSHOT,ZORAN_VSTR);
			zror(ZORAN_VDC_VIDEN,ZORAN_VDC);

			/* start single-shot grab */
			zror(ZORAN_VSTR_GRAB, ZORAN_VSTR);

			set_bit(STATE_ACTIVE, &ztv->state);
			break;
		 default:
			printk(CARD_INFO "what is this doing on the queue? (kindof=%d)\n",CARD,work->kindof);
			list_del(&work->work_item);
			if (!list_empty(&ztv->workqueue))
				goto again;	/* yeah, sure.. */
		}
		/* bye for now */
		return;
	}
DEBUG(printk(CARD_DEBUG "nothing in queue\n",CARD));

	/*
	 * What? Even the workqueue is empty? Am i really here
	 * for nothing? Did i come all that way to... do nothing?
	 */

	/* do we need to overlay? */
	if (test_bit(STATE_OVERLAY, &ztv->state))
	{
		/* are we already overlaying? */
		if (!(zrread(ZORAN_OCR) & ZORAN_OCR_OVLEN) ||
		    !(zrread(ZORAN_VDC) & ZORAN_VDC_VIDEN))
		{
DEBUG(printk(CARD_DEBUG "starting overlay\n",CARD));

			read_lock(&ztv->lock);
			zoran_set_geo(ztv,&ztv->overinfo);
			read_unlock(&ztv->lock);

			zror(ZORAN_OCR_OVLEN, ZORAN_OCR);
			zrand(~ZORAN_VSTR_SNAPSHOT,ZORAN_VSTR);
			zror(ZORAN_VDC_VIDEN,ZORAN_VDC);

			set_bit(STATE_ACTIVE, &ztv->state);
		}

		/*
		 * leave overlaying on, but turn interrupts off.
		 */
		zrand(~ZORAN_ICR_EN,ZORAN_ICR);
		return;
	}

	/* do we have any VBI idle time processing? */
	if (test_bit(STATE_VBI, &ztv->state))
	{
		struct vidinfo* item;

		/* protect the workqueue */
		write_lock(&ztv->lock);
		/* scan for available buffers and queue them */
		for (item=ztv->vbiinfo; item!=ztv->vbiinfo+ztv->nrvbis; item++)
			if (item->available)
			{
				item->available = 0;
				list_add(&item->work_item, &ztv->workqueue);
			}
		write_unlock(&ztv->lock);

		/* if we did find something, restart the engine */
		if (!list_empty(&ztv->workqueue))
			goto again;	/* gotos? hey, _i_ graduated :) */
	}

	/*
	 * Then we must be realy IDLE
	 */
DEBUG(printk(CARD_DEBUG "turning off\n",CARD));
	/* nothing further to do, disable DMA and further IRQs */
	zrand(~ZORAN_VDC_VIDEN,ZORAN_VDC);
	zrand(~ZORAN_ICR_EN,ZORAN_ICR);
	clear_bit(STATE_ACTIVE, &ztv->state);
}

/**
 * zoran_irq() - process the interrupt
 * @irq: IRQ signal,
 * @dev_id: driver data,
 * @regs: registers at the time of the interrupt.
 *
 * Does interrupt processing based on @irq, @dev_id. @regs is
 * not used.
 **/
static
void zoran_irq(int irq, void *dev_id, struct pt_regs * regs)
{
	u32 stat,estat;
	int count = 0;
	struct zoran *ztv = (struct zoran *)dev_id;

	UNUSED(irq); UNUSED(regs);
	for (;;) {
		/* get/clear interrupt status bits */
		stat=zrread(ZORAN_ISR);
		estat=stat & zrread(ZORAN_ICR);
		if (!estat)
			return;
		zrwrite(estat,ZORAN_ISR);
		IDEBUG(printk(CARD_DEBUG "estat %08x\n",CARD,estat));
		IDEBUG(printk(CARD_DEBUG " stat %08x\n",CARD,stat));

		if (estat & ZORAN_ISR_CODE)
		{
			IDEBUG(printk(CARD_DEBUG "CodReplIRQ\n",CARD));
		}
		if (estat & ZORAN_ISR_GIRQ0)
		{
			IDEBUG(printk(CARD_DEBUG "GIRQ0\n",CARD));
			if (!ztv->card->usegirq1) {
				ztv->fieldnr++;
				reap_states(ztv);
			}
		}
		if (estat & ZORAN_ISR_GIRQ1)
		{
			IDEBUG(printk(CARD_DEBUG "GIRQ1\n",CARD));
			if (ztv->card->usegirq1) {
				ztv->fieldnr++;
				reap_states(ztv);
			}
		}

		count++;
		if (count > 10)
			printk(CARD_ERR "irq loop %d (%x)\n",CARD,count,estat);
		if (count > 20)
		{
			zrwrite(0, ZORAN_ICR);
			printk(CARD_ERR "IRQ lockup, cleared int mask\n",CARD);
		}
	}
}

static
int zoran_muxsel(struct zoran* ztv, int channel, int norm)
{
	int	changed = 0;
	int	rv;

	/* map the given channel# to the cards decoder's channel */
	channel = ztv->card->video_mux[channel] & CHANNEL_MASK;

	/*
	 * First set the new channel, for AUTO we need the norm of
	 * the new channel, not the old!
	 */
	if (ztv->channel != channel) {
		rv = zr36120_i2c_call(ztv, DECODER_SET_INPUT, &channel);
//		if (rv)
//			return rv;
		ztv->channel = channel;
		changed = 1;
	}

	/* set the new video norm */
	if (changed || ztv->norm == VIDEO_MODE_AUTO || ztv->norm != norm) {
		rv = zr36120_i2c_call(ztv, DECODER_SET_NORM, &norm);
//		if (rv)
//			return rv;
		ztv->norm = norm;
	}
	return 0;
}

/* Tell the interrupt handler what to to.  */
static
void zoran_cap(struct zoran* ztv, int on)
{
DEBUG(printk(CARD_DEBUG "zoran_cap(%d) state=%x\n",CARD,on,ztv->state));

	if (on) {
		if (!test_bit(STATE_ACTIVE,&ztv->state)) {
			/* force a rescan of the queues */
DEBUG(printk(CARD_DEBUG "forcing immediate start of engine \n",CARD));
			reap_states(ztv);
		}

		ztv->running = 1;

		/*
		 * turn interrupts (back) on. The DMA will be enabled
		 * inside the irq handler when it detects a restart.
		 */
		zror(ZORAN_ICR_EN,ZORAN_ICR);
	}
	else {
		/*
		 * turn both interrupts and DMA off
		 */
		zrand(~ZORAN_VDC_VIDEN,ZORAN_VDC);
		zrand(~ZORAN_ICR_EN,ZORAN_ICR);

		ztv->running = 0;
	}
}

static
void clip_rectangle(u8 *viewmap, int x, int y, int w, int h)
{
static const u8 lmaskt[8] = { 0x00,0x01,0x03,0x07,0x0f,0x1f,0x3f,0x7f};
static const u8 rmaskt[8] = { 0xfe,0xfc,0xf8,0xf0,0xe0,0xc0,0x80,0x00};
	u8 lmask, rmask, *p;
	int W, l, r;
	int i;

DEBUG(printk(KERN_DEBUG "clip_rectangle(%d,%d,%d,%d)\n",x,y,w,h));

	if (x+w>1024) w = 1024-x;
	if (y+h>625) h = 625-y;

	l=x/8;
	r=(x+w-1)/8;
	W=r-l-1;
	lmask=lmaskt[x&7];
	rmask=rmaskt[(x+w-1)&7];
	p=viewmap+128*y+l;

	if (W>0)
		for (i=0; i<h; i++, p+=128) {
			p[0]   &= lmask;
			memset(p+1, 0, W);
			p[W+1] &= rmask;
		}
	else if (!W)
		for (i=0; i<h; i++, p+=128) {
			p[0] &= lmask;
			p[1] &= rmask;
		}
	else for (i=0; i<h; i++, p+=128)
			p[0] &= lmask | rmask;
}

#define max(a,b) ({ typeof(a) _a = a, _b = b; _a<_b ? _b : _a; })

static
void zoran_built_overlay(struct zoran* ztv, int count, struct video_clip *vcp)
{
	u8* viewmap = (u8*)ztv->overinfo.mskadr;
	ulong *mtop = ztv->overinfo.mskadr;
	int height = ztv->overinfo.h;
	int width = ztv->vidWidth;
	int ystep, i;

	/* This is the third time I fell for it: ystep is in DWORDS! */
	ystep = (ztv->vidInterlace ? 64 : 32) - (ztv->vidXshift+width+31)/32;

DEBUG(	printk(KERN_DEBUG "       overlay at %p, ystep=%d, clips=%d\n",ztv->overinfo.mskadr,ystep,count);
	if (count != VIDEO_CLIP_BITMAP) {
		struct video_clip *vp = vcp;
		for (i=0; i<count; i++) {
			printk(KERN_DEBUG "       %d: clip(%d,%d,%d,%d)\n", i,vp->x,vp->y,vp->width,vp->height);
			++vp;
		}
	})

	if(width > 1023)
		width = 1023;           /* sanity check */
	if(height > 625)
		height = 625;           /* sanity check */

	if (count == VIDEO_CLIP_BITMAP)
	{	/* bitmap was passed */
		ulong* p = (ulong*)viewmap;
		memcpy(viewmap, (u8*)vcp, VIDEO_CLIPMAP_SIZE);
		/* invert the sense of the viewmap */
		for (i=0; i<VIDEO_CLIPMAP_SIZE/sizeof(ulong); i++)
			*p ^= ~0UL;

		/****************************************************
		 * We have a problem, the visible section of the
		 * clipmap should be moved to Xshift,0.
		 */
printk(KERN_DEBUG "Problem; how to handle this map?\n");
	}
	else
	{       /* setup a nice - fully enabled - viewmap */
		memset(viewmap, ~0, VIDEO_CLIPMAP_SIZE);

		/* now disable the given clips */
		for (i=0; i<count; i++) {
			/* discard entirly off-window clips */
			if (vcp[i].x+vcp[i].width < 0 || vcp[i].x > width ||
			    vcp[i].y+vcp[i].height < 0 || vcp[i].y > height)
				continue;
			/* adjust partialy off-window clips */
			if (vcp[i].x < 0) {
				vcp[i].width += vcp[i].x;
				vcp[i].x = 0;
			}
			if (vcp[i].y < 0) {
				vcp[i].height += vcp[i].y;
				vcp[i].y  = 0;
			}
			if (vcp[i].x + vcp[i].width > width)
				vcp[i].width = width - vcp[i].x;
			if (vcp[i].y + vcp[i].height > height)
				vcp[i].height = height - vcp[i].y;
			/* well, it must be in range than... */
			clip_rectangle(viewmap, vcp[i].x+ztv->vidXshift, vcp[i].y, vcp[i].width, vcp[i].height);
		}
	}

	/* clip against Xshift, viewing window AND screen
	 * so we do not have to rely on the user program
	 */
	clip_rectangle(viewmap,ztv->vidXshift +
			       (ztv->overinfo.x+width > ztv->swidth) ?
			       (ztv->swidth - ztv->overinfo.x - width)
				: width, 0,1024,768);
	clip_rectangle(viewmap,ztv->vidXshift,
				(ztv->overinfo.y+height > ztv->sheight) ?
				(ztv->sheight - ztv->overinfo.y - height)
				: height,1024,768);
	if (ztv->vidXshift)
		clip_rectangle(viewmap, 0, 0, ztv->vidXshift,768);

	/* now tell the chip what we have done */
	zrwrite(virt_to_bus(mtop), ZORAN_MTOP);
	zrwrite(virt_to_bus(mtop+128), ZORAN_MBOT);
	zraor(ystep<<0,~ZORAN_OCR_MASKSTRIDE,ZORAN_OCR);
}

struct tvnorm 
{
	u16 Wt, Wa, Ht, Ha, HStart, VStart;
};

static struct tvnorm tvnorms[] = {
	/* PAL-BDGHI */
/*	{ 864, 720, 625, 576, 131, 21 },*/
/*00*/	{ 864, 768, 625, 576, 85, 17 },
	/* NTSC */
/*01*/	{ 858, 720, 525, 480, 121, 10 },
	/* SECAM */
/*02*/	{ 864, 720, 625, 576, 131, 21 },
	/* BW50 */
/*	{ 864, 720, 625, 576, 131, 21 },*/
/*03*/	{ 864, 768, 625, 576, 85, 17 },
	/* BW60 */
/*04*/	{ 858, 720, 525, 480, 121, 10 }
};
#define TVNORMS (sizeof(tvnorms)/sizeof(tvnorms[0]))

/*
 * Program the chip for a setup as described in the vidinfo struct.
 *
 * Side-effects: calculates vidXshift, vidInterlace,
 * vidWidth which are used in a later stage
 * to calculate the overlay mask
 *
 * This is an internal function, as such it does not check the
 * validity of the struct members... Spectaculair crashes will
 * follow /very/ quick when you're wrong and the chip right :)
 */
static
void zoran_set_geo(struct zoran* ztv, struct vidinfo* i)
{
	ulong	top, bot;
	int	stride;
	int	winWidth, winHeight;
	int	maxWidth, maxHeight, maxXOffset, maxYOffset;
	long	vfec;

DEBUG(printk(CARD_DEBUG "set_geo(rect=(%d,%d,%d,%d), norm=%d, format=%d, bpp=%d, bpl=%d, memadr=%p, busadr=%lx, overlay=%p)\n",CARD,i->x,i->y,i->w,i->h,ztv->norm,i->format,i->bpp,i->bpl,i->memadr,(ulong)i->busadr,i->mskadr));

	/*
	 * make sure the DMA transfers are inhibited during our
	 * reprogramming of the chip
	 */
	zrand(~ZORAN_VDC_VIDEN,ZORAN_VDC);

	maxWidth = tvnorms[ztv->norm].Wa;
	maxHeight = tvnorms[ztv->norm].Ha/2;
	maxXOffset = tvnorms[ztv->norm].HStart;
	maxYOffset = tvnorms[ztv->norm].VStart;

	/* setup vfec register (keep ExtFl,TopField and VCLKPol settings) */
	vfec = (zrread(ZORAN_VFEC) & (ZORAN_VFEC_EXTFL|ZORAN_VFEC_TOPFIELD|ZORAN_VFEC_VCLKPOL)) |
	       (palette2fmt[i->format].mode & (ZORAN_VFEC_RGB|ZORAN_VFEC_ERRDIF|ZORAN_VFEC_LE|ZORAN_VFEC_PACK24));

	/*
	 * Set top, bottom ptrs. Since these must be DWORD aligned,
	 * possible adjust the x and the width of the window.
	 * so the endposition stays the same. The vidXshift will make
	 * sure we are not writing pixels before the requested x.
	 */
	ztv->vidXshift = 0;
	winWidth = i->w;
	top = i->busadr + i->x*i->bpp + i->y*i->bpl;
	if (top & 3) {
		ztv->vidXshift = (top & 3) / i->bpp;
		winWidth += ztv->vidXshift;
		DEBUG(printk(KERN_DEBUG "       window-x shifted %d pixels left\n",ztv->vidXshift));
		top &= ~3;
	}

	/*
	 * bottom points to next frame but in interleaved mode we want
	 * to 'mix' the 2 frames to one capture, so 'bot' points to one
	 * (physical) line below the top line.
	 */
	bot = top + i->bpl;
	zrwrite(top,ZORAN_VTOP);
	zrwrite(bot,ZORAN_VBOT);

#if FDEBUG
/* remember the top of both buffers for debug purposes */
i->tfield = (ulong*)(i->memadr + top - i->busadr);
i->bfield = (ulong*)(i->memadr + bot - i->busadr);
#endif

	/*
	 * Make sure the winWidth is DWORD aligned too,
	 * thereby automaticly making sure the stride to the
	 * next line is DWORD aligned too (as required by spec).
	 */
	if ((winWidth*i->bpp) & 3) {
DEBUG(printk(KERN_DEBUG "       window-width enlarged by %d pixels\n",(winWidth*i->bpp) & 3));
		winWidth += (winWidth*i->bpp) & 3;
	}

	/* determine the DispMode and stride */
	if (i->h > maxHeight || i->kindof==FBUFFER_VBI) {
		/* interleaving needed for this height */
		ztv->vidInterlace = 1;
		stride = i->bpl*2 - (winWidth * i->bpp);
		winHeight = i->h/2;
	}
	else {
		/* single frame grab suffices for this height. */
		vfec |= (ZORAN_VFEC_DISPMOD | ZORAN_VFEC_DUPFLD);

		ztv->vidInterlace = 0;
		stride = i->bpl - (winWidth * i->bpp);
		winHeight = i->h;
	}

	/* safety net, sometimes bpl is too short??? */
	if (stride<0) {
DEBUG(printk(CARD_DEBUG "WARNING stride = %d\n",CARD,stride));
		stride = 0;
	}

	zraor((winHeight<<12)|(winWidth<<0),~(ZORAN_VDC_VIDWINHT|ZORAN_VDC_VIDWINWID), ZORAN_VDC);
	zraor(stride<<16,~ZORAN_VSTR_DISPSTRIDE,ZORAN_VSTR);

	/* remember vidWidth for overlay calculations */
	ztv->vidWidth = winWidth;
DEBUG(printk(KERN_DEBUG "       top=%08lx, bottom=%08lx\n",top,bot));
DEBUG(printk(KERN_DEBUG "       winWidth=%d, winHeight=%d\n",winWidth,winHeight));
DEBUG(printk(KERN_DEBUG "       maxWidth=%d, maxHeight=%d\n",maxWidth,maxHeight));
DEBUG(printk(KERN_DEBUG "       stride=%d\n",stride));

	/*
	 * determine horizontal scales and crops
	 *
	 * VBI reads are easy. They get the whole line.
	 */
	if (i->kindof == FBUFFER_VBI) {
		int Hstart = 1;
		int Hend = Hstart + winWidth - 1;
DEBUG(printk(KERN_DEBUG "       X: scale=0, start=%d, end=%d\n", Hstart, Hend));
		zraor((Hstart<<10)|(Hend<<0),~(ZORAN_VFEH_HSTART|ZORAN_VFEH_HEND),ZORAN_VFEH);
	}
	else {
		int Wa = maxWidth;
		int X = (winWidth*64+Wa-1)/Wa;
		int We = winWidth*64/X;
		int HorDcm = 64-X;
		int hcrop1 = 2*(Wa-We)/4;
		/*
		 * BUGFIX: Juha Nurmela <junki@qn-lpr2-165.quicknet.inet.fi> 
		 * found the solution to the color phase shift.
		 * See ChangeLog for the full explanation)
		 */
		int Hstart = (maxXOffset + hcrop1) | 1;
		int Hend = Hstart + We - 1;

DEBUG(printk(KERN_DEBUG "       X: scale=%d, start=%d, end=%d\n", HorDcm, Hstart, Hend));

		zraor((Hstart<<10)|(Hend<<0),~(ZORAN_VFEH_HSTART|ZORAN_VFEH_HEND),ZORAN_VFEH);
		vfec |= HorDcm<<14;

		if (HorDcm<16)
			vfec |= ZORAN_VFEC_HFILTER_1; /* no filter */
		else if (HorDcm<32)
			vfec |= ZORAN_VFEC_HFILTER_3; /* 3 tap filter */
		else if (HorDcm<48)
			vfec |= ZORAN_VFEC_HFILTER_4; /* 4 tap filter */
		else	vfec |= ZORAN_VFEC_HFILTER_5; /* 5 tap filter */
	}

	/*
	 * Determine vertical scales and crops
	 *
	 * For VBI reads we start at line 1, for others we calculate
	 * the allowable height for the norm and scale to it.
	 */
	if (i->kindof == FBUFFER_VBI) {
		int Vstart = 1;
		int Vend = Vstart + winHeight - 1;
DEBUG(printk(KERN_DEBUG "       Y: scale=0, start=%d, end=%d\n", Vstart, Vend));
		zraor((Vstart<<10)|(Vend<<0),~(ZORAN_VFEV_VSTART|ZORAN_VFEV_VEND),ZORAN_VFEV);
	}
	else {
		int Ha = maxHeight;
		int Y = (winHeight*64+Ha-1)/Ha;
		int He = winHeight*64/Y;
		int VerDcm = 64-Y;
		int vcrop1 = 2*(Ha-He)/4;
		int Vstart = maxYOffset + vcrop1;
		int Vend = Vstart + He - 1;

DEBUG(printk(KERN_DEBUG "       Y: scale=%d, start=%d, end=%d\n", VerDcm, Vstart, Vend));
		zraor((Vstart<<10)|(Vend<<0),~(ZORAN_VFEV_VSTART|ZORAN_VFEV_VEND),ZORAN_VFEV);
		vfec |= VerDcm<<8;
	}

DEBUG(printk(KERN_DEBUG "       F: format=%d(=%s)\n",i->format,palette2fmt[i->format].name));

	/* setup the requested format */
	zrwrite(vfec, ZORAN_VFEC);
}

static
void zoran_common_open(struct zoran* ztv, int flags)
{
	UNUSED(flags);

	/* already opened? */
	if (ztv->users++ != 0)
		return;

	/* unmute audio */
	/* /what/ audio? */

	ztv->state = 0;

	/* setup the encoder to the initial values */
	ztv->picture.colour=254<<7;
	ztv->picture.brightness=128<<8;
	ztv->picture.hue=128<<8;
	ztv->picture.contrast=216<<7;
	zr36120_i2c_call(ztv, DECODER_SET_PICTURE, &ztv->picture);

	/* default to the composite input since my camera is there */
	zoran_muxsel(ztv, 0, VIDEO_MODE_PAL);

	INIT_LIST_HEAD(&ztv->workqueue);
}

static
void zoran_common_close(struct zoran* ztv)
{
	if (--ztv->users != 0)
		return;

	/* mute audio */
	/* /what/ audio? */

	/* stop the chip */
	zoran_cap(ztv, 0);
}

/*
 * Open a zoran card. Right now the flags are just a hack
 */
static int zoran_open(struct video_device *dev, int flags)
{
	struct zoran *ztv = (struct zoran*)dev;
	struct vidinfo* item;

	VDEBUG(printk(CARD_DEBUG "open(dev,%d)\n",CARD,flags));

	/*********************************************
	 * We really should be doing lazy allocing...
	 *********************************************/
	if (!ztv->overinfo.mskadr) {
		ztv->overinfo.mskadr = (void*)kmalloc(VIDEO_CLIPMAP_SIZE, GFP_KERNEL);
		if (!ztv->overinfo.mskadr)
			return -ENOBUFS;
	}

	/* get framebuffers and give them a DMAable address */
	for (item=ztv->capinfo; item!=ztv->capinfo+ztv->nrcaps; item++)
		if (!item->memadr)
		{
			item->loaded = 0;
			item->available = 1;
			item->memadr = zmalloc(ztv->szcap);
			if (!item->memadr) {
				/* could not get a buffer, bail out */
				while (item != ztv->capinfo) {
					item--;
					zfree(item->memadr, ztv->szcap);
					item->memadr = 0;
					item->busadr = 0;
				}
				kfree(ztv->overinfo.mskadr);
				ztv->overinfo.mskadr = 0;
				return -ENOBUFS;
			}

			/* determine the DMAable address */
			item->busadr = virt_to_bus(item->memadr);

			/* at this time there better be a framebuffer */
			memset(item->memadr,0,ztv->szcap);
		}

	/* do the common part of all open's */
	zoran_common_open(ztv, flags);

	return 0;
}

static
void zoran_close(struct video_device* dev)
{
	struct zoran *ztv = (struct zoran*)dev;
	struct vidinfo* item;

	VDEBUG(printk(CARD_DEBUG "close(dev)\n",CARD));

	/* driver specific closure */
	clear_bit(STATE_OVERLAY, &ztv->state);

	zoran_common_close(ztv);

        /*
         *      This is sucky but right now I can't find a good way to
         *      be sure its safe to free the buffer. We wait 5-6 fields
         *      which is more than sufficient to be sure.
         */
        set_current_state(TASK_UNINTERRUPTIBLE);
        schedule_timeout(HZ/10);        /* Wait 1/10th of a second */

	/* free the allocated framebuffers */
	for (item=ztv->capinfo; item!=ztv->capinfo+ztv->nrcaps; item++)
	{
		if (item->memadr) {
			zfree(item->memadr, ztv->szcap);
			item->memadr = 0;
		}
	}
	if (ztv->overinfo.mskadr) {
		kfree( ztv->overinfo.mskadr );
		ztv->overinfo.mskadr = 0;
	}
}

/*
 * This read function could be used reentrant in a SMP situation.
 *
 * This is made possible by the spinlock which is kept till we
 * found and marked a buffer for our own use. The lock must than
 * be released as soon as possible to prevent lock contention.
 */
static
long zoran_read(struct video_device* dev, char* buf, unsigned long count, int nonblock)
{
	struct zoran *ztv = (struct zoran*)dev;
	unsigned long max;
	struct vidinfo* item;
	DECLARE_WAITQUEUE(wait, current);

	VDEBUG(printk(CARD_DEBUG "zoran_read(%p,%ld,%d)\n",CARD,buf,count,nonblock));
	/* add myself to the buffer wait queue */
	add_wait_queue(&ztv->capbq, &wait);
	for (;;) {
		set_current_state(TASK_INTERRUPTIBLE);

		/* find the first available buffer  */
		write_lock_irq(&ztv->lock);
		for (item=ztv->capinfo; !item->available && item!=ztv->capinfo+ztv->nrcaps; ++item)
			/* empty */;
		if (item!=ztv->capinfo+ztv->nrcaps)
			break;

		/* nothing found, i'm upset now... */
		write_unlock_irq(&ztv->lock);

		/* lets wait a little... */
		if (nonblock) {
			set_current_state(TASK_RUNNING);
			remove_wait_queue(&ztv->capbq, &wait);
			return -EWOULDBLOCK;
		}
		if (signal_pending(current)) {
			set_current_state(TASK_RUNNING);
			remove_wait_queue(&ztv->capbq, &wait);
			return -ERESTARTSYS;
		}
		schedule();
	}
	set_current_state(TASK_RUNNING);
	remove_wait_queue(&ztv->capbq, &wait);

	/*
	 * mark the unused buffer as unavailable.
	 * After that we can release the lock, since it
	 * will not be picked up by free-buffer locators
	 */
	item->available = 0;
	write_unlock_irq(&ztv->lock);
	item->loaded = 0;
	item->w = 320;
	item->h = 240;
	item->format = VIDEO_PALETTE_RGB24;
	item->bpp = palette2fmt[item->format].bpp;
	item->bpl = item->w * item->bpp;
	list_add_tail(&item->work_item,&ztv->workqueue);

	/* tell the state machine we want it filled /NOW/ */
	zoran_cap(ztv, 1);

	/* wait till this buffer gets fetched */
	if (wait_event_interruptible(ztv->capcq,item->loaded)) {
		count = -ERESTARTSYS;
		goto out;
	}

	/* Yes! we got data! */
	max = item->bpl * item->h;
	if (count > max)
		count = max;

	/* well, to sum this copying stuff up: Y-UK! */
	if (copy_to_user((void*)buf, item->memadr, count))
		count = -EFAULT;

	/* tell everyone this buffer became free */
out:	item->available = 1;
	wake_up_interruptible(&ztv->capbq);

	/* goodbye */
	VDEBUG(printk(CARD_DEBUG "zoran_read() returns %lu\n",CARD,count));
	return count;
}

static
unsigned int zoran_poll(struct video_device *dev, struct file *file, poll_table *wait)
{
	struct zoran *ztv = (struct zoran *)dev;
	struct vidinfo* item;
	unsigned int mask = 0;

	poll_wait(file, &ztv->capcq, wait);

	for (item=ztv->capinfo; item!=ztv->capinfo+ztv->nrcaps; item++)
		if (!item->available && item->loaded)
		{
			mask |= (POLLIN | POLLRDNORM);
			break;
		}

//	VDEBUG(printk(CARD_DEBUG "zoran_poll()=%x\n",CARD,mask));

	return mask;
}

static
int zoran_ioctl(struct video_device* dev, unsigned int cmd, void *arg)
{
	struct zoran* ztv = (struct zoran*)dev;

	switch (cmd) {
	 case VIDIOCGCAP:
	 {
		struct video_capability c;
		VDEBUG(printk(CARD_DEBUG "VIDIOCGCAP\n",CARD));

		strcpy(c.name,ztv->video_dev.name);
		c.type = VID_TYPE_CAPTURE|
			 VID_TYPE_OVERLAY|
			 VID_TYPE_CLIPPING|
			 VID_TYPE_FRAMERAM|
			 VID_TYPE_SCALES;
		if (ztv->have_tuner)
			c.type |= VID_TYPE_TUNER;
		if (ztv->have_decoder) {
			c.channels = ztv->card->video_inputs;
			c.audios = ztv->card->audio_inputs;
		} else
			/* no decoder -> no channels */
			c.channels = c.audios = 0;
		c.maxwidth = tvnorms[ztv->norm].Wa;
		c.maxheight = tvnorms[ztv->norm].Ha;
		c.minwidth = 32;
		c.minheight = 32;
		if (copy_to_user(arg,&c,sizeof(c)))
			return -EFAULT;
		break;
	 }

	 case VIDIOCGCHAN:
	 {
		struct video_channel v;
		int mux;
		if (copy_from_user(&v, arg,sizeof(v)))
			return -EFAULT;
		VDEBUG(printk(CARD_DEBUG "VIDIOCGCHAN(%d)\n",CARD,v.channel));

		/* too many inputs? no decoder -> no channels */
		if (!ztv->have_decoder || v.channel >= ztv->card->video_inputs)
			return -EINVAL;

		v.flags=0;
		if (ztv->card->audio_inputs)
			v.flags |= VIDEO_VC_AUDIO;
		v.tuners=0;
		v.type=VIDEO_TYPE_CAMERA;
		v.norm=ztv->norm;

		/* now determine the name of the channel */
		mux = ztv->card->video_mux[v.channel];
		if (mux & IS_TUNER) {
			/* lets assume only one tuner, yes? */
			strcpy(v.name,"Television");
			v.type = VIDEO_TYPE_TV;
			if (ztv->have_tuner) {
				v.flags |= VIDEO_VC_TUNER;
				v.tuners = 1;
			}
		}
		else if (mux & IS_SVHS)
			sprintf(v.name,"S-Video-%d",v.channel);
		else
			sprintf(v.name,"CVBS-%d",v.channel);

		if (copy_to_user(arg,&v,sizeof(v)))
			return -EFAULT;
		break;
	 }
	 case VIDIOCSCHAN:
	 {	/* set video channel */
		struct video_channel v;
		if (copy_from_user(&v, arg,sizeof(v)))
			return -EFAULT;
		VDEBUG(printk(CARD_DEBUG "VIDIOCSCHAN(%d,%d)\n",CARD,v.channel,v.norm));

		/* too many inputs? no decoder -> no channels */
		if (!ztv->have_decoder || v.channel >= ztv->card->video_inputs)
			return -EINVAL;

		if (v.norm > TVNORMS)
			return -EOPNOTSUPP;

		/* make it happen, nr1! */
		return zoran_muxsel(ztv,v.channel,v.norm);
	 }

	 case VIDIOCGTUNER:
	 {
		struct video_tuner v;
		if (copy_from_user(&v, arg,sizeof(v)))
			return -EFAULT;
		VDEBUG(printk(CARD_DEBUG "VIDIOCGTUNER(%d)\n",CARD,v.tuner));

		/* Only no or one tuner for now */
		if (!ztv->have_tuner || v.tuner)
			return -EINVAL;

		strcpy(v.name,"Television");
		v.rangelow  = 0;
		v.rangehigh = ~0;
		v.flags     = VIDEO_TUNER_PAL|VIDEO_TUNER_NTSC|VIDEO_TUNER_SECAM;
		v.mode      = ztv->norm;
		v.signal    = 0xFFFF; /* unknown */

		if (copy_to_user(arg,&v,sizeof(v)))
			return -EFAULT;
		break;
	 }
	 case VIDIOCSTUNER:
	 {
		struct video_tuner v;
		if (copy_from_user(&v, arg, sizeof(v)))
			return -EFAULT;
		VDEBUG(printk(CARD_DEBUG "VIDIOCSTUNER(%d,%d)\n",CARD,v.tuner,v.mode));

		/* Only no or one tuner for now */
		if (!ztv->have_tuner || v.tuner)
			return -EINVAL;

		/* and it only has certain valid modes */
		if( v.mode != VIDEO_MODE_PAL &&
		    v.mode != VIDEO_MODE_NTSC &&
		    v.mode != VIDEO_MODE_SECAM)
			return -EOPNOTSUPP;

		/* engage the new video norm */
		return zoran_muxsel(ztv,ztv->channel,v.mode);
	 }

	 case VIDIOCGPICT:
	 {
		struct video_picture p = ztv->picture;
		VDEBUG(printk(CARD_DEBUG "VIDIOCGPICT\n",CARD));
		p.depth = ztv->depth;
		switch (p.depth) {
		 case 15: p.palette=VIDEO_PALETTE_RGB555;
			  break;
		 case 16: p.palette=VIDEO_PALETTE_RGB565;
			  break;
		 case 24: p.palette=VIDEO_PALETTE_RGB24;
			  break;
		 case 32: p.palette=VIDEO_PALETTE_RGB32;
			  break;
		}
		if (copy_to_user(arg, &p, sizeof(p)))
			return -EFAULT;
		break;
	 }
	 case VIDIOCSPICT:
	 {
		struct video_picture p;
		if (copy_from_user(&p, arg,sizeof(p)))
			return -EFAULT;
		VDEBUG(printk(CARD_DEBUG "VIDIOCSPICT(%d,%d,%d,%d,%d,%d,%d)\n",CARD,p.brightness,p.hue,p.colour,p.contrast,p.whiteness,p.depth,p.palette));

		/* depth must match with framebuffer */
		if (p.depth != ztv->depth)
			return -EINVAL;

		/* check if palette matches this bpp */
		if (p.palette>NRPALETTES ||
		    palette2fmt[p.palette].bpp != ztv->overinfo.bpp)
			return -EINVAL;

		write_lock_irq(&ztv->lock);
		ztv->overinfo.format = p.palette;
		ztv->picture = p;
		write_unlock_irq(&ztv->lock);

		/* tell the decoder */
		zr36120_i2c_call(ztv, DECODER_SET_PICTURE, &p);
		break;
	 }

	 case VIDIOCGWIN:
	 {
		struct video_window vw;
		VDEBUG(printk(CARD_DEBUG "VIDIOCGWIN\n",CARD));
		read_lock(&ztv->lock);
		vw.x      = ztv->overinfo.x;
		vw.y      = ztv->overinfo.y;
		vw.width  = ztv->overinfo.w;
		vw.height = ztv->overinfo.h;
		vw.chromakey= 0;
		vw.flags  = 0;
		if (ztv->vidInterlace)
			vw.flags|=VIDEO_WINDOW_INTERLACE;
		read_unlock(&ztv->lock);
		if (copy_to_user(arg,&vw,sizeof(vw)))
			return -EFAULT;
		break;
	 }
	 case VIDIOCSWIN:
	 {
		struct video_window vw;
		struct video_clip *vcp = NULL;
		int on;

		if (copy_from_user(&vw,arg,sizeof(vw)))
			return -EFAULT;
		VDEBUG(printk(CARD_DEBUG "VIDIOCSWIN(%d,%d,%d,%d,%x,%d)\n",CARD,vw.x,vw.y,vw.width,vw.height,vw.flags,vw.clipcount));

		if (vw.flags ||
		    vw.width<32 || vw.width>tvnorms[ztv->norm].Wa ||
		    vw.height<32 || vw.height>tvnorms[ztv->norm].Ha)
			return -EINVAL;

		if (vw.clipcount != VIDEO_CLIP_BITMAP &&
		    (vw.clipcount<0 || vw.clipcount>256))
			return -EDOM;   /* Too many! */

		/*
		 *      Do any clips. Note that it now possble
		 *	to get a clipmap when clipcount has a
		 *	'magic' value. In a clipmap, '1' denotes
		 *	a clipped pixel, not a visible pixel!
		 */
		if (vw.clipcount == VIDEO_CLIP_BITMAP) {
			vcp = vmalloc(VIDEO_CLIPMAP_SIZE);
			if (vcp == NULL)
				return -ENOMEM;
			if (copy_from_user(vcp,vw.clips,VIDEO_CLIPMAP_SIZE)) {
				vfree(vcp);
				return -EFAULT;
			}
		} else if (vw.clipcount) {
			vcp = vmalloc(sizeof(struct video_clip)*vw.clipcount);
			if (vcp == NULL)
				return -ENOMEM;
			if (copy_from_user(vcp,vw.clips,sizeof(struct video_clip)*vw.clipcount)) {
				vfree(vcp);
				return -EFAULT;
			}
		}

		on = ztv->running;
		if (on)
			zoran_cap(ztv, 0);

		/* by now we are committed to the new data... */
		write_lock_irq(&ztv->lock);
		ztv->overinfo.x = vw.x;
		ztv->overinfo.y = vw.y;
		ztv->overinfo.w = vw.width;
		ztv->overinfo.h = vw.height;
		write_unlock_irq(&ztv->lock);

		/* setup and calculate the chip parameters */
		zoran_set_geo(ztv, &ztv->overinfo);
		/* only then built the requested clipping zones */
		zoran_built_overlay(ztv, vw.clipcount, vcp);
		if (vcp)
			vfree(vcp);

		/* if we were on, restart the video engine */
		if (on)
			zoran_cap(ztv, 1);
		break;
	 }

	 case VIDIOCCAPTURE:
	 {
		int v;
		if (get_user(v,(int*)arg))
			return -EFAULT;
		VDEBUG(printk(CARD_DEBUG "VIDIOCCAPTURE(%d)\n",CARD,v));

		if (v==0) {
			clear_bit(STATE_OVERLAY, &ztv->state);
			zoran_cap(ztv, 1);
		}
		else {
			/* is VIDIOCSFBUF, VIDIOCSWIN done? */
			if (ztv->overinfo.busadr==0 || ztv->overinfo.w<32 || ztv->overinfo.h<32)
				return -EINVAL;

			set_bit(STATE_OVERLAY, &ztv->state);
			zoran_cap(ztv, 1);
		}
		break;
	 }

	 case VIDIOCGFBUF:
	 {
		struct video_buffer v;
		VDEBUG(printk(CARD_DEBUG "VIDIOCGFBUF\n",CARD));
		read_lock(&ztv->lock);
		v.base   = (void *)ztv->overinfo.busadr;
		v.height = ztv->sheight;
		v.width  = ztv->swidth;
		v.depth  = ztv->depth;
		v.bytesperline = ztv->overinfo.bpl;
		read_unlock(&ztv->lock);
		if(copy_to_user(arg, &v,sizeof(v)))
			return -EFAULT;
		break;
	 }
	 case VIDIOCSFBUF:
	 {
		struct video_buffer v;
#if LINUX_VERSION_CODE >= 0x020100
			if(!capable(CAP_SYS_ADMIN) && !capable(CAP_SYS_ADMIN))
#else
			if(!suser())
#endif
			return -EPERM;
		if (copy_from_user(&v, arg,sizeof(v)))
			return -EFAULT;
		VDEBUG(printk(CARD_DEBUG "VIDIOCSFBUF(%p,%d,%d,%d,%d)\n",CARD,v.base, v.width,v.height,v.depth,v.bytesperline));

		if (v.depth!=15 && v.depth!=16 && v.depth!=24 && v.depth!=32)
			return -EINVAL;
		if (v.bytesperline<1)
			return -EINVAL;
		if (ztv->running)
			return -EBUSY;
		write_lock_irq(&ztv->lock);
		ztv->overinfo.busadr  = (ulong)v.base;
		ztv->sheight      = v.height;
		ztv->swidth       = v.width;
		ztv->depth        = v.depth;		/* bits per pixel */
		ztv->overinfo.bpp = ((v.depth+1)&0x38)/8;/* bytes per pixel */
		ztv->overinfo.bpl = v.bytesperline;	/* bytes per line */
		write_unlock_irq(&ztv->lock);
		break;
	 }

	 case VIDIOCKEY:
	 {
		/* Will be handled higher up .. */
		break;
	 }

	 case VIDIOCSYNC:
	 {
		struct vidinfo* item;
		int i;
		if (get_user(i,(int*)arg))
			return -EFAULT;
		VDEBUG(printk(CARD_DEBUG "VIDEOCSYNC(%d)\n",CARD,i));

		if (i<0 || i>=ztv->nrcaps)
			return -EINVAL;

		item = ztv->capinfo+i;
		if (item->available)
			return -EINVAL;

		/* test and wait for completion */
		if (wait_event_interruptible(ztv->capcq,item->loaded))
			return -ERESTARTSYS;

		/* tell ppl we have a spare buffer again */
		item->available = 1;
		wake_up_interruptible(&ztv->capbq);

		VDEBUG(printk(CARD_DEBUG "VIDEOCSYNC(%d) returns\n",CARD,i));
		break;
	 }

	 case VIDIOCMCAPTURE:
	 {
		struct video_mmap vm;
		struct vidinfo* item;
		if (copy_from_user(&vm,arg,sizeof(vm)))
			return -EFAULT;
		VDEBUG(printk(CARD_DEBUG "VIDIOCMCAPTURE(%d,(%d,%d),%d)\n",CARD,vm.frame,vm.width,vm.height,vm.format));

		/* sanity checks */
		if (vm.frame>=ztv->nrcaps ||
		    vm.format>NRPALETTES ||
		    palette2fmt[vm.format].bpp == 0 ||
		    vm.width<32 || vm.width>tvnorms[ztv->norm].Wa ||
		    vm.height<32 || vm.height>tvnorms[ztv->norm].Ha ||
		    vm.width*vm.height*palette2fmt[vm.format].bpp > ztv->szcap)
			return -EINVAL;

		/* we are allowed to take over available buffers
		 * Again we must free the lock ASAP. */
		item = &ztv->capinfo[vm.frame];
		write_lock_irq(&ztv->lock);
		if (!item->available) {
			write_unlock_irq(&ztv->lock);
			return -EBUSY;
		}
		item->available = 0;	/* its mine, Mine! */
		write_unlock_irq(&ztv->lock);
		item->loaded = 0;
		item->w = vm.width;
		item->h = vm.height;
		item->format = vm.format;
		item->bpp = palette2fmt[item->format].bpp;
		item->bpl = item->w * item->bpp;
		list_add_tail(&item->work_item,&ztv->workqueue);

		zoran_cap(ztv, 1);
		break;
	 }

	 case VIDIOCGMBUF:
	 {
		struct video_mbuf mb;
		int i;
		VDEBUG(printk(CARD_DEBUG "VIDIOCGMBUF\n",CARD));
		mb.size = ztv->nrcaps * ztv->szcap;
		mb.frames = ztv->nrcaps;
		for (i=0; i<ztv->nrcaps; i++)
			mb.offsets[i] = i*ztv->szcap;
		if(copy_to_user(arg, &mb,sizeof(mb)))
			return -EFAULT;
		break;
	 }

	 case VIDIOCGUNIT:
	 {
		struct video_unit vu;
		VDEBUG(printk(CARD_DEBUG "VIDIOCGUNIT\n",CARD));
		vu.video = ztv->video_dev.minor;
		vu.vbi = ztv->vbi_dev.minor;
		vu.radio = VIDEO_NO_UNIT;
		vu.audio = VIDEO_NO_UNIT;
		vu.teletext = VIDEO_NO_UNIT;
		if(copy_to_user(arg, &vu,sizeof(vu)))
			return -EFAULT;
		break;
	 }

	 case VIDIOCGFREQ:
	 {
		unsigned long v = ztv->tuner_freq;
		if (copy_to_user(arg,&v,sizeof(v)))
			return -EFAULT;
		VDEBUG(printk(CARD_DEBUG "VIDIOCGFREQ\n",CARD));
		break;
	 }
	 case VIDIOCSFREQ:
	 {
		unsigned long v;
		if (copy_from_user(&v, arg, sizeof(v)))
			return -EFAULT;
		VDEBUG(printk(CARD_DEBUG "VIDIOCSFREQ\n",CARD));

		if (ztv->have_tuner) {
			int fixme = v;
			if (zr36120_i2c_call(ztv, TUNER_SET_TVFREQ, &fixme) < 0)
				return -EAGAIN;
		}
		ztv->tuner_freq = v;
		break;
	 }

	 default:
		return -ENOIOCTLCMD;
	}
	return 0;
}

static
int zoran_mmap(struct video_device* dev, const char* adr, unsigned long size)
{
	struct zoran* ztv = (struct zoran*)dev;
	unsigned long start = (unsigned long)adr;
	struct vidinfo* item;

	VDEBUG(printk(CARD_DEBUG "zoran_mmap(0x%p,%ld)\n",CARD,adr,size));

	/* sanity checks */
	if (size > ztv->nrcaps*ztv->szcap)
		return -EINVAL;

	/* start mapping the whole shabang to user memory */
	size = PAGE_ALIGN(size);

	for (item=ztv->capinfo; size && item!=ztv->capinfo+ztv->nrcaps; item++) {
		char* pos = item->memadr;
		unsigned long siz = PAGE_ALIGN(ztv->szcap);
		while (siz && size) {
			unsigned long page = virt_to_phys(pos);
			if (remap_page_range(start, page, PAGE_SIZE, PAGE_SHARED))
				return -EAGAIN;
			start += PAGE_SIZE;
			pos += PAGE_SIZE;
			siz -= PAGE_SIZE;
			size -= PAGE_SIZE;
		}
	}
	return 0;
}

static struct video_device zr36120_template=
{
	owner: THIS_MODULE,
	name: "UNSET",
	type: VID_TYPE_TUNER|VID_TYPE_CAPTURE|VID_TYPE_OVERLAY,
	hardware: VID_HARDWARE_ZR36120,
	open: zoran_open,
	close: zoran_close,
	read: zoran_read,
	poll: zoran_poll,
	ioctl: zoran_ioctl,
	mmap: zoran_mmap,
	minor: -1
};

/* ----------------------------------------------------------------------- */

static
int vbi_open(struct video_device *dev, int flags)
{
	struct zoran *ztv = (struct zoran*)dev->priv;
	struct vidinfo* item;

	VDEBUG(printk(CARD_DEBUG "vbi_open(dev,%d)\n",CARD,flags));

	/*
	 * During VBI device open, we continuously grab VBI-like
	 * data in the vbi buffer when we have nothing to do.
	 * Only when there is an explicit request for VBI data
	 * (read call) we /force/ a read.
	 */

	/* allocate buffers */
	for (item=ztv->vbiinfo; item!=ztv->vbiinfo+ztv->nrvbis; item++)
		if (!item->memadr)
		{
			item->loaded = 0;
			item->available = 1;
			item->memadr = zmalloc(ZORAN_VBI_BUFSIZE);
			if (!item->memadr) {
				/* could not get a buffer, bail out */
				while (item != ztv->vbiinfo) {
					item--;
					zfree(item->memadr, ZORAN_VBI_BUFSIZE);
					item->memadr = 0;
					item->busadr = 0;
				}
				return -ENOBUFS;
			}

			/* determine the DMAable address */
			item->busadr = virt_to_bus(item->memadr);
		}

	/* do the common part of all open's */
	zoran_common_open(ztv, flags);

	set_bit(STATE_VBI, &ztv->state);
	/* start read-ahead */
	zoran_cap(ztv, 1);

	return 0;
}

static
void vbi_close(struct video_device *dev)
{
	struct zoran *ztv = (struct zoran*)dev->priv;
	struct vidinfo* item;

	VDEBUG(printk(CARD_DEBUG "vbi_close(dev)\n",CARD));

	/* driver specific closure */
	clear_bit(STATE_VBI, &ztv->state);

	zoran_common_close(ztv);

        /*
         *      This is sucky but right now I can't find a good way to
         *      be sure its safe to free the buffer. We wait 5-6 fields
         *      which is more than sufficient to be sure.
         */
        set_current_state(TASK_UNINTERRUPTIBLE);
        schedule_timeout(HZ/10);        /* Wait 1/10th of a second */

	for (item=ztv->vbiinfo; item!=ztv->vbiinfo+ztv->nrvbis; item++)
	{
		if (item->memadr) {
			zfree(item->memadr, ZORAN_VBI_BUFSIZE);
			item->memadr = 0;
		}
	}
}

/*
 * This read function could be used reentrant in a SMP situation.
 *
 * This is made possible by the spinlock which is kept till we
 * found and marked a buffer for our own use. The lock must
 * be released as soon as possible to prevent lock contention.
 */
static
long vbi_read(struct video_device* dev, char* buf, unsigned long count, int nonblock)
{
	struct zoran *ztv = (struct zoran*)dev->priv;
	unsigned long max;
	DECLARE_WAITQUEUE(wait, current);
	struct vidinfo* unused = 0;
	struct vidinfo* done = 0;

	VDEBUG(printk(CARD_DEBUG "vbi_read(0x%p,%ld,%d)\n",CARD,buf,count,nonblock));

	/* add myself to the buffer wait queue */
	add_wait_queue(&ztv->vbibq, &wait);
	for (;;) {
		struct vidinfo* item;
		set_current_state(TASK_INTERRUPTIBLE);

		/* find the first available buffer  */
		write_lock_irq(&ztv->lock);
		for (item=ztv->vbiinfo; item!=ztv->vbiinfo+ztv->nrvbis; item++) {
			if (!unused && item->available)
				unused = item;
			if (!done && item->loaded)
				done = item;
		}
		if (done || unused)
			break;

		/* no more free buffers, i hate this! */
		write_unlock_irq(&ztv->lock);

		/* lets wait a little... */
		if (nonblock) {
			set_current_state(TASK_RUNNING);
			remove_wait_queue(&ztv->vbibq, &wait);
			return -EWOULDBLOCK;
		}
		if(signal_pending(current)) {
			set_current_state(TASK_RUNNING);
			remove_wait_queue(&ztv->vbibq, &wait);
			return -ERESTARTSYS;
		}
		schedule();
	}
	set_current_state(TASK_RUNNING);
	remove_wait_queue(&ztv->vbibq, &wait);

	/* Do we have 'ready loaded' data? */
	if (!done) {
		/* no? than this will take a while... */
		if (nonblock) {
			write_unlock_irq(&ztv->lock);
			count = -EWOULDBLOCK;
			goto out;
		}
		
		/* mark the unused buffer as wanted.
		 * After setting available to 0, it will
		 * no longer be picked up by free buffer
		 * locators, so its really mine
		 */
		unused->available = 0;
		write_unlock_irq(&ztv->lock);
		unused->loaded = 0;
		list_add_tail(&unused->work_item,&ztv->workqueue);

		/* tell the state machine we want it filled /NOW/ */
		zoran_cap(ztv, 1);

		/* wait till this buffer gets fetched */
		if (wait_event_interruptible(ztv->vbicq,unused->loaded)) {
			count = -ERESTARTSYS;
			goto out;
		}
		done = unused;
	}
	else write_unlock_irq(&ztv->lock);

	/* Yes! we got data! */
	max = done->bpl * done->h;
	if (count > max)
		count = max;

	/* check if the user gave us enough room to write the data */
	if (!access_ok(VERIFY_WRITE, buf, count)) {
		count = -EFAULT;
		goto out;
	}

	/*
	 * Now transform/strip the data from YUV to Y-only
	 * NB. Assume the Y is in the LSB of the YUV data.
	 */
	{
	unsigned char* optr = buf;
	unsigned char* eptr = buf+count;

	/* are we beeing accessed from an old driver? */
	if (count == 2*19*2048) {
		/*
		 * Extreme HACK, old VBI programs expect 2048 points
		 * of data, and we only got 864 orso. Double each 
		 * datapoint and clear the rest of the line.
		 * This way we have appear to have a
		 * sample_frequency of 29.5 Mc.
		 */
		int x,y;
		unsigned char* iptr = done->memadr;
		for (y=0; optr<eptr && y<done->h; y++)
		{
			/* copy to doubled data to userland */
			for (x=0; optr+1<eptr && x<done->w; x++)
			{
				unsigned char a = iptr[x*2];
				*optr++ = a;
				*optr++ = a;
			}
			/* and clear the rest of the line */
			for (x*=2; optr<eptr && x<done->bpl; x++)
				*optr++ = 0;
			/* next line */
			iptr += done->bpl;
		}
	}
	else {
		/*
		 * Other (probably newer) programs asked
		 * us what geometry we are using, and are
		 * reading the correct size. We still have
		 * to transform from YUV to single-Y...
		 */
		int x,y;
		unsigned char* iptr = done->memadr;
		for (y=0; optr<eptr && y<done->h; y++)
		{
			/* copy to doubled data to userland */
			for (x=0; optr<eptr && x<done->w; x++)
				*optr++ = iptr[x*2];
			/* and clear the rest of the line */
			for (;optr<eptr && x<done->bpl; x++)
				*optr++ = 0;
			/* next line */
			iptr += done->bpl;
		}
	}

	/* API compliance:
	 * place the framenumber (half fieldnr) in the last long
	 */
	((ulong*)eptr)[-1] = done->fieldnr/2;
	}

	/* keep the engine running */
	zoran_cap(ztv, 1);

	/* tell listeners a buffer just became free */
out:	done->available = 1;
	wake_up_interruptible(&ztv->vbibq);

	/* goodbye */
	VDEBUG(printk(CARD_DEBUG "vbi_read() returns %lu\n",CARD,count));
	return count;
}

static
unsigned int vbi_poll(struct video_device *dev, struct file *file, poll_table *wait)
{
	struct zoran *ztv = (struct zoran*)dev->priv;
	struct vidinfo* item;
	unsigned int mask = 0;

	poll_wait(file, &ztv->vbicq, wait);

	for (item=ztv->vbiinfo; item!=ztv->vbiinfo+ztv->nrvbis; item++)
		if (item->loaded)
		{
			mask |= (POLLIN | POLLRDNORM);
			break;
		}

//	VDEBUG(printk(CARD_DEBUG "vbi_poll()=%x\n",CARD,mask));

	return mask;
}

static
int vbi_ioctl(struct video_device *dev, unsigned int cmd, void *arg)
{
	struct zoran* ztv = (struct zoran*)dev->priv;

	switch (cmd) {
	 case VIDIOCGCAP:
	 {
		struct video_capability b;
		VDEBUG(printk(CARD_DEBUG "VIDIOCGCAP\n",CARD));

		strcpy(b.name,ztv->vbi_dev.name);
		b.type = VID_TYPE_TELETEXT;
		if (ztv->have_tuner)
			b.type |= VID_TYPE_TUNER;
		b.channels = 0;
		b.audios = 0;
		b.maxwidth = 0;
		b.maxheight = 0;
		b.minwidth = 0;
		b.minheight = 0;
		if(copy_to_user(arg,&b,sizeof(b)))
			return -EFAULT;
		break;
	 }

	 case VIDIOCGFREQ:
	 case VIDIOCSFREQ:
		return zoran_ioctl((struct video_device*)ztv,cmd,arg);

	 case VIDIOCGVBIFMT:
	 {
		/* warning, hardcode PAL constant follows */
		struct vbi_format f;
		VDEBUG(printk(CARD_DEBUG "VIDIOCGVBIINFO\n",CARD));
		f.sampling_rate = 14750000UL;
//		f.offset = 0;
		f.samples_per_line = ztv->vbiinfo[0].w;
		f.sample_format = VIDEO_PALETTE_RAW;
		f.start[0] = f.start[1] = ztv->vbiinfo[0].y;
		f.start[1] += 312;
		f.count[0] = f.count[1] = ztv->vbiinfo[0].h;
		f.flags = VBI_INTERLACED;
		if (copy_to_user(arg,&f,sizeof(f)))
			return -EFAULT;
		break;
	 }
	 case VIDIOCSVBIFMT:
	 {
		struct vbi_format f;
		int i;
		if (copy_from_user(&f, arg,sizeof(f)))
			return -EFAULT;
		VDEBUG(printk(CARD_DEBUG "VIDIOCSVBIINFO(%d,%d,%d,%d,%d,%d,%d,%x)\n",CARD,f.sampling_rate,f.samples_per_line,f.sample_format,f.start[0],f.start[1],f.count[0],f.count[1],f.flags));

		/* lots of parameters are fixed... (PAL) */
		if (f.sampling_rate != 14750000UL ||
		    f.samples_per_line > 864 ||
		    f.sample_format != VIDEO_PALETTE_RAW ||
		    f.start[0] != f.start[1]-312 ||
		    f.count[0] != f.count[1] ||
		    f.start[0]+f.count[0] >= 288 ||
		    f.flags != VBI_INTERLACED)
			return -EINVAL;

		write_lock_irq(&ztv->lock);
		ztv->vbiinfo[0].y = f.start[0];
		ztv->vbiinfo[0].w = f.samples_per_line;
		ztv->vbiinfo[0].h = f.count[0];
		ztv->vbiinfo[0].bpl = f.samples_per_line*ztv->vbiinfo[0].bpp;
		/* all frames are created equal */
		for (i=1; i<ztv->nrvbis; i++)
			ztv->vbiinfo[i] = ztv->vbiinfo[0];
		write_unlock_irq(&ztv->lock);
		break;
	 }
	 default:
		return -ENOIOCTLCMD;
	}
	return 0;
}

static struct video_device vbi_template=
{
	name: "UNSET",
	owner: THIS_MODULE,
	type: VID_TYPE_CAPTURE|VID_TYPE_TELETEXT,
	hardware: VID_HARDWARE_ZR36120,
	open: vbi_open,
	close: vbi_close,
	read: vbi_read,
	poll: vbi_poll,
	ioctl: vbi_ioctl,
	minor: -1
};

/* ----------------------------------------------------------------------- */

static int __devinit zoran_init(struct zoran* ztv, int card);
static void __devexit zoran_remove(struct pci_dev* dev);

/*
 * The PCI subsystem found a chip we registered for. Be jealous :)
 */
static
int __devinit zoran_probe(struct pci_dev* dev,
			  const struct pci_device_id* pci_id)
{
	int result;
	struct zoran *ztv;
	unsigned char revision;

	/* Ok, a ZR36120/ZR36125 found! */
	ztv = &zorans[zoran_cards];
	ztv->dev = dev;

	/* is the user-supplied cardtype valid? */
	if (cardtype[zoran_cards]>=NRTVCARDS) {
		printk(KERN_INFO "zoran-%d: invalid cardtype(%d) detected\n",zoran_cards,cardtype[zoran_cards]);
		return -EINVAL;
	}
	if (nrcaps[zoran_cards]<2 || nrcaps[zoran_cards]>ZORAN_CAP_MAXFRAMES) {
		printk(KERN_INFO "zoran-%d: invalid number of capture buffers (%u)\n",zoran_cards,nrcaps[zoran_cards]);
		return -EINVAL;
	}
	if (szcap[zoran_cards]<1024 || szcap[zoran_cards]>ZORAN_CAP_BUFSIZE) {
		printk(KERN_INFO "zoran-%d: invalid size for capture buffer (%u)\n",zoran_cards,szcap[zoran_cards]);
		return -EINVAL;
	}
	if (nrvbis[zoran_cards]<2 || nrvbis[zoran_cards]>ZORAN_VBI_MAXFRAMES) {
		printk(KERN_INFO "zoran-%d: invalid number of vbibuffers (%u)\n",zoran_cards,nrvbis[zoran_cards]);
		return -EINVAL;
	}

	if (pci_enable_device(dev))
		return -EIO;

	if (!request_mem_region(pci_resource_start(dev, 0),
			        pci_resource_len(dev, 0),
				"zoran"))
		return -EBUSY;

	pci_read_config_byte(dev, PCI_CLASS_REVISION, &revision);
	printk(KERN_INFO "zoran-%d: Zoran %x (rev %d) "
		"bus: %d, devfn: %d, irq: %d, memory: 0x%08lx.\n",
		zoran_cards, dev->device, revision,
		dev->bus->number, dev->devfn, dev->irq,
		pci_resource_start(dev, 0));

	ztv->zoran_mem = ioremap(pci_resource_start(dev, 0),
				pci_resource_len(dev, 0));
	DEBUG(printk(KERN_DEBUG "zoran: mapped-memory at 0x%p\n",ztv->zoran_mem));

	/* now go and get yourself the interrupt */
	result = request_irq(dev->irq, zoran_irq,
			     SA_SHIRQ|SA_INTERRUPT,"zoran",(void *)ztv);
	if (result == -EINVAL)
	{
		printk(KERN_ERR "zoran-%d: Bad irq number or handler\n",zoran_cards);
		goto fail1;
	}
	if (result == -EBUSY) {
		printk(KERN_ERR "zoran-%d: IRQ %d busy, change your PnP config in BIOS\n",dev->irq,zoran_cards);
		goto fail1;
	}
	if (result < 0)
		goto fail1;

	/* Enable bus-mastering */
	pci_set_master(dev);
	pci_set_drvdata(dev,ztv);

	if (zoran_init(ztv,zoran_cards) < 0) {
		zoran_remove(dev);
		result = -EIO;
		goto fail;
	}

	printk(KERN_INFO "         using %u buffers with %uk (%dk total) for capture and %u buffers for vbi.\n",
		ztv->nrcaps,ztv->szcap/1024,ztv->nrcaps*ztv->szcap/1024,
		ztv->nrvbis);

	zoran_cards++;
	return 0;

fail:
	free_irq(dev->irq,ztv);

fail1:
	release_mem_region(pci_resource_start(dev, 0),
			   pci_resource_len(dev,0));

	pci_set_drvdata(dev,NULL);
	return result;
}

static
int __devinit zoran_init(struct zoran* ztv, int card)
{
	int	i;

	/* soft reset the zoran */
	zrand(~ZORAN_PCI_SOFTRESET,ZORAN_PCI);
	udelay(10);
	zror(ZORAN_PCI_SOFTRESET,ZORAN_PCI);
	udelay(10);

	/* zoran chip specific details */
	ztv->card = tvcards+cardtype[card];
	ztv->norm = VIDEO_MODE_PAL;
	ztv->channel = 0;
	ztv->tuner_freq = 0;
	ztv->have_decoder = 0;
	ztv->have_tuner = 0;
	ztv->tuner_type = 0;
	ztv->running = 0;
	ztv->users = 0;
	ztv->nrcaps = nrcaps[card];
	ztv->szcap = szcap[card];
	ztv->nrvbis = nrvbis[card];
	ztv->lock = RW_LOCK_UNLOCKED;
	ztv->fieldnr = 0;

	/* videocard details */
	ztv->swidth = 800;
	ztv->sheight = 600;
	ztv->depth = 16;

	/* State details */
	ztv->overinfo.kindof = FBUFFER_OVERLAY;
	ztv->overinfo.loaded = 0;
	ztv->overinfo.available = 1;
	ztv->overinfo.x = 0;
	ztv->overinfo.y = 0;
	ztv->overinfo.w = 768; /* 640 */
	ztv->overinfo.h = 576; /* 480 */
	ztv->overinfo.format = VIDEO_PALETTE_RGB565;
	ztv->overinfo.bpp = palette2fmt[ztv->overinfo.format].bpp;
	ztv->overinfo.bpl = ztv->overinfo.bpp*ztv->swidth;
	ztv->overinfo.busadr = 0;
	ztv->overinfo.memadr = 0;
	ztv->overinfo.mskadr = 0;
	for (i=0; i<ZORAN_CAP_MAXFRAMES; i++) {
		ztv->capinfo[i] = ztv->overinfo;
		ztv->capinfo[i].kindof = FBUFFER_CAP;
	}
	init_waitqueue_head(&ztv->capcq);
	init_waitqueue_head(&ztv->capbq);

	/* VBI details */
	ztv->vbiinfo[0] = ztv->overinfo;
	ztv->vbiinfo[0].kindof = FBUFFER_VBI;
	ztv->vbiinfo[0].w = 864;
	ztv->vbiinfo[0].h = 44;
	ztv->vbiinfo[0].format = VIDEO_PALETTE_YUYV;
	ztv->vbiinfo[0].bpp = palette2fmt[ztv->vbiinfo[0].format].bpp;
	ztv->vbiinfo[0].bpl = 1024*ztv->vbiinfo[0].bpp;
	for (i=1; i<ZORAN_VBI_MAXFRAMES; i++)
		ztv->vbiinfo[i] = ztv->vbiinfo[0];
	init_waitqueue_head(&ztv->vbicq);
	init_waitqueue_head(&ztv->vbibq);

	if (triton1)
		zrand(~ZORAN_VDC_TRICOM, ZORAN_VDC);

	/* external FL determines TOP frame */
	if (ztv->card->ext_fl) {
		/* positive signal => topfield? */
		if (ztv->card->fld_pos)
			zror(ZORAN_VFEC_EXTFL|ZORAN_VFEC_TOPFIELD, ZORAN_VFEC); 
		else
			zror(ZORAN_VFEC_EXTFL, ZORAN_VFEC); 
	}

	/* set HSpol */
	if (ztv->card->hsync_pos)
		zrwrite(ZORAN_VFEH_HSPOL, ZORAN_VFEH);
	/* set VSpol */
	if (ztv->card->vsync_pos)
		zrwrite(ZORAN_VFEV_VSPOL, ZORAN_VFEV);

	/* Set the proper General Purpuse register bits */
	/* implicit: no softreset, 0 waitstates */
	zrwrite(ZORAN_PCI_SOFTRESET|(ztv->card->gpdir<<0),ZORAN_PCI);
	/* implicit: 3 duration and recovery PCI clocks on guest 0-3 */
	zrwrite(ztv->card->gpval<<24,ZORAN_GUEST);

	/* clear interrupt status */
	zrwrite(~0, ZORAN_ISR);

	/*
	 * Now add the template and register the device unit
	 */
	ztv->video_dev = zr36120_template;
	sprintf(ztv->video_dev.name,"zoran-%d",card);
	ztv->video_dev.priv = ztv;
	if (video_register_device(&ztv->video_dev, VFL_TYPE_GRABBER) < 0)
		return -1;

	ztv->vbi_dev = vbi_template;
	strcpy(ztv->vbi_dev.name, ztv->video_dev.name);
	ztv->vbi_dev.priv = ztv;
	if (video_register_device(&ztv->vbi_dev, VFL_TYPE_VBI) < 0) {
		video_unregister_device(&ztv->video_dev);
		return -1;
	}

	/*
	 * connect the i2c adapter
	 */
	if (zr36120_i2c_init(ztv)) {
		video_unregister_device(&ztv->video_dev);
		video_unregister_device(&ztv->vbi_dev);
		return -1;
	}

	/* set interrupt mask - the PIN enable will be set later */
	zrwrite(ZORAN_ICR_GIRQ0|ZORAN_ICR_GIRQ1|ZORAN_ICR_CODE, ZORAN_ICR);

	printk(KERN_INFO "%s: installed %s\n",ztv->video_dev.name,ztv->card->name);
	return 0;
}

/*
 * Deactivate and remove the given zoran card from the driver's
 * working set.
 */
static
void __devexit zoran_remove(struct pci_dev* dev)
{
	struct zoran *ztv = pci_get_drvdata(dev);

	/* unregister i2c_bus */
	zr36120_i2c_exit(ztv);

	/* turn off all capturing, DMA and IRQs by resetting the beast */
	zrand(~ZORAN_PCI_SOFTRESET,ZORAN_PCI);
	udelay(10);
	zror(ZORAN_PCI_SOFTRESET,ZORAN_PCI);
	udelay(10);

	/* better safe than sorry: disable interrupts! */
	zrwrite(0, ZORAN_ICR);
	zrwrite(0xffffffffUL,ZORAN_ISR);

	/* unregister video4linux */
	if (ztv->video_dev.minor != -1)
		video_unregister_device(&ztv->video_dev);
	if (ztv->vbi_dev.minor != -1)
		video_unregister_device(&ztv->vbi_dev);

	/* free the used irq */
	free_irq(ztv->dev->irq,ztv);

	/* unmap and free memory */
	if (ztv->zoran_mem)
		iounmap(ztv->zoran_mem);

	release_mem_region(pci_resource_start(dev, 0), pci_resource_len(dev,0));
	pci_set_drvdata(dev,NULL);

}

/*----------------------------------------------------------------------*
 * ZORAN chipset detector						*
 * shamelessly stolen from bttv.c					*
 * Reason for beeing here: we need to detect if we are running on a	*
 * Triton based chipset, and if so, enable a certain bit		*
 *----------------------------------------------------------------------*/
static
void __devinit handle_chipset(void)
{
	struct pci_dev *dev = NULL;

	/* Just in case some nut set this to something dangerous */
	if (triton1)
		triton1 = ZORAN_VDC_TRICOM;

	while ((dev = pci_find_device(PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82437, dev)))
	{
		printk(KERN_INFO "zoran: Host bridge 82437FX Triton PIIX\n");
		triton1 = ZORAN_VDC_TRICOM;
	}
}

/* ----------------------------------------------------------------------- */

static struct pci_device_id zoran_pci_tbl[] __devinitdata = {
        {PCI_VENDOR_ID_ZORAN, PCI_DEVICE_ID_ZORAN_36120,
         PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
        {0,}
};

MODULE_DEVICE_TABLE(pci, zoran_pci_tbl);

static struct pci_driver zoran_pci_driver = {
        name:     "zoran",
        id_table: zoran_pci_tbl,
        probe:    zoran_probe,
        remove:   zoran_remove,
};

int __devinit zr36120_init(void)
{
	handle_chipset();

	return pci_module_init(&zoran_pci_driver);
}

void __devexit zr36120_exit(void)
{
	pci_unregister_driver(&zoran_pci_driver);
}

module_init(zr36120_init);
module_exit(zr36120_exit);
