/*
    zr36120_i2c.c - Generic I2C interface for zr36120.c

    Copyright (C) 1998-2000 Pauline Middelink <middelink@polyware.nl>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <linux/config.h>
#include <linux/types.h>
#define __NO_VERSION__
#include <linux/module.h>
#include <linux/version.h>
#include <linux/delay.h>
#include <asm/io.h>
#include <asm/uaccess.h>

#include "linux/video_decoder.h"
#include "linux/tuner.h"
#include "zr36120.h"

/* ----------------------------------------------------------------------- */
/* I2C bitbanger functions						   */
/* ----------------------------------------------------------------------- */

static void zr36120_bit_setscl(void *data, int state)
{
	struct zoran *ztv = data;
	int bit = ztv->card->swapi2c ? ZORAN_I2C_SDA : ZORAN_I2C_SCL;

	if (state)
		ztv->i2c_state |= bit;
	else
		ztv->i2c_state &= ~bit;

	zrwrite(ztv->i2c_state, ZORAN_I2C);
	zrread(ZORAN_I2C);      // for PCI flush
}

static void zr36120_bit_setsda(void *data, int state)
{
	struct zoran *ztv = data;
	int bit = ztv->card->swapi2c ? ZORAN_I2C_SCL : ZORAN_I2C_SDA;

	if (state)
		ztv->i2c_state |= bit;
	else
		ztv->i2c_state &= ~bit;

	zrwrite(ztv->i2c_state, ZORAN_I2C);
	zrread(ZORAN_I2C);      // for PCI flush
}

static int zr36120_bit_getscl(void *data)
{
	struct zoran *ztv = data;

	if (ztv->card->swapi2c)
		return zrread(ZORAN_I2C) & ZORAN_I2C_SDA;
	return !!(zrread(ZORAN_I2C) & ZORAN_I2C_SCL);
}

static int zr36120_bit_getsda(void *data)
{
	struct zoran *ztv = data;

	if (ztv->card->swapi2c)
		return !!(zrread(ZORAN_I2C) & ZORAN_I2C_SCL);
	return zrread(ZORAN_I2C) & ZORAN_I2C_SDA;
}

static struct i2c_algo_bit_data i2c_algo_template = {
	NULL,
	zr36120_bit_setsda,
	zr36120_bit_setscl,
	zr36120_bit_getsda,
	zr36120_bit_getscl,
	20, 10, 100,
};

/* ----------------------------------------------------------------------- */
/* I2C adapter functions                                                   */
/* ----------------------------------------------------------------------- */

static void inc_use(struct i2c_adapter *adap)
{
	MOD_INC_USE_COUNT;
}

static void dec_use(struct i2c_adapter *adap)
{
	MOD_DEC_USE_COUNT;
}

static int attach_inform(struct i2c_client *client)
{
	struct zoran *ztv = (struct zoran*)client->adapter->data;
	struct video_decoder_capability dc;
	int i;

	for (i = 0; i < I2C_CLIENTS_MAX; i++) {
		if (!ztv->i2c_clients[i] ||
		    ztv->i2c_clients[i]->driver->id == client->driver->id) {
			ztv->i2c_clients[i] = client;
			break;
		}
	}
//	if (ztv->tuner_type != -1)
//		i2c_control(client,TUNER_SET_TYPE,(ulong)&ztv->tuner_type);

	/* check if it's a decoder */
	if (client->driver &&
	    client->driver->command &&
	    client->driver->command(client,DECODER_GET_CAPABILITIES,(ulong)&dc) == 0)
	{
		/* Test if the decoder can do VBI transfers */
		ztv->have_decoder = 0;
		if (dc.flags & VIDEO_DECODER_RAW)
			ztv->have_decoder |= 4;
		/* or TTX transfers */
		else if (dc.flags & VIDEO_DECODER_TTX)
			ztv->have_decoder |= 2;
		/* else it's a stupid thing... */
		else
			ztv->have_decoder |= 1;

		DEBUG(printk(CARD_DEBUG "capabilities %u %d %d, decoder=%d\n",CARD,dc.flags,dc.inputs,dc.outputs,ztv->have_decoder));
	}

#if 0
	switch (client->driver->id) {
	 case I2 _DRIVERID_TUNER:
		ztv->have_tuner = 1;
		DEBUG(printk(CARD_INFO "tuner attached\n",CARD));
		if (ztv->tuner_type >= 0)
		{
			if (zoran_i2c_call(ztv,TUNER_SET_TYPE,&ztv->tuner_type)<0)
			DEBUG(printk(CARD_INFO "attach_inform; tuner wont be set to type %d\n",CARD,ztv->tuner_type));
		}
		break;
	}
#endif

	DEBUG(printk(CARD_INFO "i2c attach [%s]\n",CARD,client->name));
	return 0;
}

static int detach_inform(struct i2c_client *client)
{
	struct zoran *ztv = (struct zoran*)client->adapter->data;
	int i;

	DEBUG(printk(CARD_INFO "i2c detach [%s]\n",CARD,client->name));

#if 0
	switch (client->driver->id) {
	 case I2C_DRIVERID_VIDEODECODER:
		ztv->have_decoder = 0;
		DEBUG(printk(CARD_INFO "decoder detached\n",CARD));
		break;
	 case I2C_DRIVERID_TUNER:
		ztv->have_tuner = 0;
		DEBUG(printk(CARD_INFO "tuner detached\n",CARD));
		break;
	}
#endif	// 0

	for (i = 0; i < I2C_CLIENTS_MAX; i++) {
		if (ztv->i2c_clients[i] &&
		    ztv->i2c_clients[i]->driver->id == client->driver->id) {
			ztv->i2c_clients[i] = NULL;
			break;
		}
	}
	return 0;
}

static struct i2c_adapter i2c_adap_template = {
	name:              "none",
	id:                I2C_HW_B_BT848,
	inc_use:           inc_use,
	dec_use:           dec_use,
	client_register:   attach_inform,
	client_unregister: detach_inform,
};


/* ----------------------------------------------------------------------- */
/* I2C functions							   */
/* ----------------------------------------------------------------------- */

int zr36120_i2c_init(struct zoran *ztv)
{
	/*
	 * i2c template
	 */
	/* i2c bit adapter */
	ztv->i2c_adap = i2c_adap_template;
	ztv->i2c_algo = i2c_algo_template;

	strcpy(ztv->i2c_adap.name, ztv->video_dev.name);
	ztv->i2c_algo.data = ztv;
	ztv->i2c_adap.data = ztv;
	ztv->i2c_adap.algo_data = &ztv->i2c_algo;

	/* give the output a defined value */
	zr36120_bit_setscl(ztv,1);
	zr36120_bit_setsda(ztv,1);

	return i2c_bit_add_bus(&ztv->i2c_adap);
}

int zr36120_i2c_exit(struct zoran *ztv)
{
	/* unregister i2c_bus */
	return i2c_bit_del_bus(&ztv->i2c_adap);
}

int zr36120_i2c_call(struct zoran *ztv, unsigned int cmd, void *arg)
{
	int i;

	for (i = 0; i < I2C_CLIENTS_MAX; i++) {
		if (!ztv->i2c_clients[i] ||
		    !ztv->i2c_clients[i]->driver->command)
			continue;

		if (ztv->i2c_clients[i]->driver->command(ztv->i2c_clients[i],cmd,(ulong)arg) != 0)
			/*return 0*/;
	}
	return -1;      // not handled
}
