/*
    zr36120_mem.c - Generic memory support for zr36120.c

    Copyright (C) 1998-2000 Pauline Middelink <middelink@polyware.nl>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <linux/config.h>
#include <linux/version.h>
#include <linux/slab.h>
#include <linux/wrapper.h>
#ifdef CONFIG_BIGPHYS_AREA
#include <linux/bigphysarea.h>
#endif

#include "zr36120.h"
#include "zr36120_mem.h"

/*******************************/
/* Memory management functions */
/*******************************/

static
inline int __get_order(unsigned long size)
{
	int order = 0;
	size = (size+PAGE_SIZE-1)/PAGE_SIZE;
	while (size) {
		size /= 2;
		order++;
	}
	return order;
}

void* zmalloc(unsigned long size)
{
	void* mem;
	size = PAGE_ALIGN(size);
#ifdef CONFIG_BIGPHYS_AREA
	mem = bigphysarea_alloc_pages(size/PAGE_SIZE, 1, GFP_KERNEL);
#else
	/*
	 * The following function got a lot of memory at boottime,
	 * so we know its always there...
	 */
	mem = (void*)__get_free_pages(GFP_USER|GFP_DMA,__get_order(size));
#endif
	if (mem) {
		ulong adr = (ulong)mem;
		while (size>0) {
			mem_map_reserve(virt_to_page(adr));
			adr += PAGE_SIZE;
			size -= PAGE_SIZE;
		}
	}
	return mem;
}

void zfree(void* mem, unsigned long size)
{
	if (mem) {
		ulong adr = (ulong)mem;
		ulong siz = PAGE_ALIGN(size);
		while (siz) {
			mem_map_unreserve(virt_to_page(adr));
			adr += PAGE_SIZE;
			siz -= PAGE_SIZE;
		}
#ifdef CONFIG_BIGPHYS_AREA
		bigphysarea_free_pages(mem);
#else
		free_pages((unsigned long)mem,__get_order(size));
#endif
	}
}
